@@ -6,7 +6,7 @@ use Carp qw/croak/;
 use POSIX qw/ceil/;
 
 
-our $VERSION = '0.10';
+our $VERSION = '0.7';
 
 require XSLoader;
 XSLoader::load('Session::Token', $VERSION);
@@ -15,13 +15,6 @@ XSLoader::load('Session::Token', $VERSION);
 my $default_alphabet = join('', ('0'..'9', 'a'..'z', 'A'..'Z',));
 my $default_entropy = 128;
 
-my $is_windows;
-
-if ($^O =~ /mswin/i) {
-  require Crypt::Random::Source::Strong::Win32;
-  $is_windows = 1;
-}
-
 
 sub new {
   my ($class, %args) = @_;
@@ -40,23 +33,17 @@ sub new {
   }
 
   if (!defined $seed) {
-    if ($is_windows) {
-      my $windows_rng_source = Crypt::Random::Source::Strong::Win32->new;
-      $seed = $windows_rng_source->get(1024);
-      die "Win32 RNG source didn't provide 1024 bytes" unless length($seed) == 1024;
-    } else {
-      my ($fh, $err1, $err2);
+    my ($fh, $err1, $err2);
 
-      open($fh, '<:raw', '/dev/urandom') || ($err1 = $!);
-      open($fh, '<:raw', '/dev/arandom') || ($err2 = $!)
-        unless defined $fh;
+    open($fh, '<', '/dev/urandom') || ($err1 = $!);
+    open($fh, '<', '/dev/arandom') || ($err2 = $!)
+      unless defined $fh;
 
-      if (!defined $fh) {
-        croak "unable to open /dev/urandom ($err1) or /dev/arandom ($err2)";
-      }
-
-      sysread($fh, $seed, 1024) == 1024 || croak "unable to read from random device: $!";
+    if (!defined $fh) {
+      croak "unable to open /dev/urandom ($err1) or /dev/arandom ($err2)";
     }
+
+    sysread($fh, $seed, 1024) == 1024 || croak "unable to read from random device: $!";
   }
 
 
@@ -142,7 +129,7 @@ Session::Token - Secure, efficient, simple random session token generation
 
 =head2 Custom entropy in bits
 
-    my $token = Session::Token->new(entropy => 256)->get;
+    my $token = Session::Token->new(entropy => 256);
     ## WdLiluxxZVkPUHsoqnfcQ1YpARuj9Z7or3COA4HNNAv
 
 =head2 Custom alphabet and length
@@ -155,15 +142,15 @@ Session::Token - Secure, efficient, simple random session token generation
 
 This module provides a secure, efficient, and simple interface for creating session tokens, password reset codes, temporary passwords, random identifiers, and anything else you can think of.
 
-When a Session::Token object is created, 1024 bytes will be read from C</dev/urandom> (Linux, Solaris, most BSDs), C</dev/arandom> (some older BSDs), or with L<Crypt::Random::Source::Strong::Win32> (Windows). These bytes will be used to seed the L<ISAAC-32|http://www.burtleburtle.net/bob/rand/isaacafa.html> pseudo random number generator.
+When a Session::Token object is created, 1024 bytes will be read from C</dev/urandom> (Linux, Solaris, most BSDs) or C</dev/arandom> (some older BSDs). These bytes will be used to seed the L<ISAAC-32|http://www.burtleburtle.net/bob/rand/isaacafa.html> pseudo random number generator.
 
 Once a generator is created, you can repeatedly call the C<get> method on the generator object and it will return new tokens.
 
-B<IMPORTANT>: If your application calls C<fork>, make sure that any generators are re-created in one of the processes after the fork since forking will duplicate the generator state and otherwise both parent and child processes will go on to produce identical tokens.
+B<IMPORTANT>: If your application calls C<fork>, make sure that the generators are re-created in one of the processes after the fork since forking will duplicate the generator state and both parent and child processes will go on to produce identical token streams.
 
 ISAAC is a cryptographically secure PRNG that improves on the well known RC4 algorithm in some important areas. For instance, it doesn't have short cycles like RC4 does. A theoretical shortest possible cycle in ISAAC is C<2**40>, although no cycles this short have ever been found (and probably don't exist at all). On average, ISAAC cycles are a ridiculous C<2**8295>.
 
-After the generator context is created, no system calls are used to generate tokens. This is one way that Session::Token helps with efficiency. This is only important for certain use cases (generally not web sessions).
+After the generator context is created, no system calls are used to generate tokens. This is one way that C<Session::Token> helps with efficiency. This is only important for certain use cases (generally not web sessions).
 
 In a server application the most important reason you should use the "keep generator around" mode instead of creating Session::Token objects every time you need a token is that in this mode generating a new token cannot fail due to a full descriptor table. Creating new generators for every token can fail for this reason. Programs that re-use the generator are also more efficient and are less likely to cause problems in C<chroot> environments.
 
@@ -228,23 +215,23 @@ In summary, the default token length of exactly 22 characters is a consequence o
 
 Many token generation libraries, especially ones that implement custom alphabets, make the mistake of generating a random value, computing its modulus over the size of an alphabet, and then using this modulus to index into the alphabet to determine an output character.
 
-Why is this bad? Consider the alphabet C<"abc">. The ideal output probability distribution for each character in the token is:
+Why is this bad? Consider the alphabet C<"abc">. An ideal output probability distribution for each character in the token is:
 
     P(a) = 1/3
     P(b) = 1/3
     P(c) = 1/3
 
-Assume we have a uniform random number source that generates values in the set C<[0,1,2,3]> (most PRNGs provide sequences of bits, in other words power-of-2 size sets). If we use the naïve modulus algorithm described above, C<0> maps to C<a>, C<1> maps to C<b>, C<2> maps to C<c>, and C<3> I<also> maps to C<a>. Instead of the even distribution above, we have the following biased distribution:
+Assume we have a uniform random number source that generates values in the set C<[0,1,2,3]> (most PRNGs provide sequences of bits, in other words power-of-2 set sizes). If we use the naïve modulus algorithm described above, C<0> maps to C<a>, C<1> maps to C<b>, C<2> maps to C<c>, and C<3> I<also> maps to C<a>. Instead of the even distribution above, we have the following biased distribution:
 
     P(a) = 2/4 = 1/2
     P(b) = 1/4
     P(c) = 1/4
 
-Session::Token eliminates this bias in the above case by only using C<0>, C<1>, and C<2>, and throwing away all C<3>s (also see the C<t/no-mod-bias.t> test).
+L<Session::Token> eliminates this bias in the above case by only using C<0>, C<1>, and C<2>, and throwing away all C<3>s (also see the C<t/no-mod-bias.t> test).
 
 Of course throwing away a portion of random data is slightly inefficient. In the worst case scenario of an alphabet with 129 characters, for each output byte this module consumes on average C<1.9845> bytes from the random number generator. This inefficiency isn't a problem because ISAAC is extremely fast.
 
-Note that if your application issues biased tokens then some tokens are more likely than other tokens which provides a starting point for token guessing. If the tokens are unbiased then all tokens are equally likely and there is no starting point.
+Note that if your application issues biased tokens, then some tokens are more likely than other tokens, providing a decent starting point for token guessing. If the tokens are unbiased, then there is not obvious starting point since all tokens are equally likely.
 
 
 
@@ -254,12 +241,12 @@ If your alphabet contains the same character two or more times, this character w
 
 However, if you wish to introduce bias this library doesn't try to stop you. (Maybe it should issue a warning?)
 
-    Session::Token->new(alphabet => '0000001', length => 5000)->get; # don't do this
+    Session::Token->new(alphabet => '0000001', length => 100000)->get; # don't do this
     ## -> 0000000000010000000110000000000000000000000100...
 
 Due to a limitation discussed below, alphabets larger than 256 aren't currently supported so your bias can't get very granular.
 
-Aside: If you have a constant-biased output stream like the above example produces then you can re-construct an un-biased bit sequence with the von neumann algorithm. This works by comparing pairs of bits. If the pair consists of identical bits, it is discarded. Otherwise the order of the different bits is used to determine an output bit, ie C<00> and C<11> are discarded but C<01> and C<10> are mapped to output bits of C<0> and C<1> respectively. This only works if the bias in each bit is constant (like all characters in a Session::Token are).
+Aside: If you have a biased output stream like the above example then you can re-construct an un-biased bit sequence with the von neumann algorithm. This works by comparing pairs of bits. If the bits are identical, they are discarded. Otherwise the order of the different bits is used to determine the output bit, ie C<00> and C<11> are discarded but C<01> and C<10> are mapped to output bits of C<0> and C<1> respectively. This only works if the bias in each input bit is constant (like in the above example).
 
 
 
@@ -287,7 +274,7 @@ However if you know what you're doing, you can pass in a custom seed as a 1024 b
 
     my $gen = Session::Token(seed => "\x00" x 1024);
 
-This is done in the test-suite, but obviously don't do this in regular applications because the generated tokens will always be the same.
+This is done in the test-suite, but obviously don't do this in regular applications because the generated token sequences will always be the same.
 
 One valid reason for seeding is if you have some reason to believe that there isn't enough entropy in your kernel's randomness pool and therefore you don't trust C</dev/urandom>. In this case you should acquire your own seed data from somewhere trustworthy (maybe C</dev/random> or a previously stored trusted seed).
 
@@ -297,7 +284,9 @@ One valid reason for seeding is if you have some reason to believe that there is
 
 It might be a good idea if this library could detect forks and re-seed in the child process.
 
-There is currently no way to extract the seed from a Session::Token object. Note when implementing this: The saved seed must either store the current state of the ISAAC round as well as the 1024 byte C<randsl> array or else do some kind of minimum fast forwarding in order to protect against a partially duplicated keystream bug.
+There is currently no way to extract the seed from a Session::Token object. Note when implementing this: The saved seed must either store the current state of the isaac round as well as the 1024 byte C<randsl> array or else do some kind of minimum fast forwarding in order to protect against a partially duplicated keystream bug.
+
+Windows isn't currently supported. Meh.
 
 
 
@@ -309,9 +298,9 @@ There are lots of different modules for generating random data.
 
 Like this module, perl's C<rand()> function implements a PRNG in user-space seeded from C</dev/urandom>. However, perl C<rand()> is seeded with a mere 4 bytes from C</dev/urandom> and the perldoc doesn't seem to specify a PRNG algorithm, so I prefer not to use C<rand()> for session tokens.
 
-L<Data::Token> is the first thing I saw when I looked around on CPAN. It has an inflexible and unspecified (?) alphabet. It tries to get its source of unpredictability from UUIDs and then hashes these UUIDs with SHA1. I think this is bad design because some standard UUID formats aren't designed to be unpredictable at all. Knowing a target's MAC address and the rough time the token was issued may help you predict a reduced area of token-space to concentrate guessing attacks upon. I don't know if Data::Token uses these types of UUIDs or the potentially secure "version 4" UUIDs, but because this wasn't addressed in the documentation and because of an apparent misapplication of hash functions (if you really had a good random UUID type, there would be no need to hash), I don't feel good about using this module.
+L<Data::Token> is the first thing I saw when I looked around on CPAN. It has an inflexible and unspecified (?) alphabet. It tries to get its source of unpredictability from UUIDs and then hashes these UUIDs with SHA1. I think this is bad design because some UUID standards aren't designed to be unpredictable at all. Knowing a target's MAC address and the rough time the token was issued may help you predict a reduced area of token-space to concentrate guessing attacks upon. I don't know if Data::Token uses these types of UUIDs or the (potentially secure) good random types, but because this wasn't addressed in the documentation and because of an apparent misapplication of hash functions (if you really had a good random UUID type, there would be no reason to hash), I don't feel good about using this module.
 
-There are several decent random number generators like L<Math::Random::Secure>, L<Crypt::URandom> &c, but they usually don't implement alphabets and some of them require you open C</dev/urandom> for every chunk of random bytes. Note that Math::Random::Secure does prevent mod bias in its random integers though.
+There are several decent random number generators like L<Math::Random::Secure>, L<Crypt::URandom> &c, but they usually don't implement alphabets and some of them require you open C</dev/urandom> for every chunk of random bytes. Note that Math::Random::Secure does prevent mod bias for its random integers though.
 
 L<String::Random> is a cool module with a neat regexp-like language for specifying random tokens which is more flexible than alphabets. However, inspecting the code indicates that it uses perl's C<rand()>. Also, the lack of performance, bias, and security discussion in the docs made me decide to not use this otherwise very interesting module.
 
